package nxtplugin;

import java.io.IOException;

import lejos.nxt.LightSensor;
import lejos.nxt.Motor;
import lejos.nxt.SensorPort;
import lejos.nxt.SoundSensor;
import lejos.nxt.TouchSensor;
import lejos.nxt.UltrasonicSensor;
import lejos.nxt.remote.NXTCommand;
import lejos.nxt.remote.RemoteMotor;
import lejos.pc.comm.NXTComm;
import lejos.pc.comm.NXTCommException;
import lejos.pc.comm.NXTInfo;
import lejos.robotics.navigation.SimpleNavigator;
import lejos.robotics.navigation.TachoPilot;

public class RemoteNXT implements Runnable, Comparable {
	protected NXTComm nxtComm;
	protected NXTInfo nxtInfo;
	protected NXTDebug debug;

	protected RemoteMotor[] remoteMotors = new RemoteMotor[3];
	protected SensorPort[] sensorPorts = new SensorPort[4];
	protected LightSensor[] lightSensors = new LightSensor[4];
	protected SoundSensor[] soundSensors = new SoundSensor[4];
	protected TouchSensor[] touchSensors = new TouchSensor[4];
	protected UltrasonicSensor[] ultrasonicSensors = new UltrasonicSensor[4];
	protected SimpleNavigator nav;

	protected float wheelDiameter = 5.6f;
	protected float trackWidth = 11.2f;
	
	protected boolean tryReconnect = true;

	public RemoteNXT(NXTComm nxtComm, NXTInfo nxtInfo, NXTDebug debug) {
		this.nxtComm = nxtComm;
		this.nxtInfo = nxtInfo;
		this.debug = debug;
		NXTCommand.getSingleton().setNXTComm(nxtComm);

		remoteMotors = new RemoteMotor[] { Motor.A, Motor.B, Motor.C };
		sensorPorts = new SensorPort[] { SensorPort.S1, SensorPort.S2, SensorPort.S3, SensorPort.S4 };

		remoteMotors[0].resetTachoCount();
		remoteMotors[1].resetTachoCount();
		nav = new SimpleNavigator(new TachoPilot(wheelDiameter, trackWidth, remoteMotors[0], remoteMotors[1]));
	}
	
	public int compareTo(Object o) {
		return nxtInfo.name.compareTo(((RemoteNXT)o).nxtInfo.name);
	}

	public void run() {
		while(tryReconnect) {
			byte[] reply;
			try {
				reply = nxtComm.sendRequest(new byte[] {NXTCommand.DIRECT_COMMAND_REPLY, NXTCommand.KEEP_ALIVE}, 7);
				if (reply.length == 0) {
					throw new IOException();
				}
				else {
					debug.addAlive(this);
				}
			} catch (IOException e1) {
				debug.logEvent("Connection to " + nxtInfo.name + " lost!");
				try {
					nxtComm.close();
				} catch (IOException e2) {
				}
				
				if (tryReconnect) {
					debug.addLost(this);
					
					debug.logEvent("Trying to reconnect to " + nxtInfo.name + " . . .");
					try {
						nxtComm.open(nxtInfo);
						debug.logEvent("Succsessfully reconnected to " + nxtInfo.name + "!");
						
						debug.addAlive(this);
					} catch (NXTCommException e) {
						debug.logEvent(nxtInfo.name + " not available!");
					}
				}
			}
			
			// wait some time
			try {
				Thread.sleep(6000);
			} catch (InterruptedException e) {
			}
		}
	}

	public String toString() {
		return nxtInfo.name;
	}
}
